/*
  Aplicaţia de faţă implică două biblioteci javaScript: 'Ymacs' şi framework-ul 'DynarchLIB'
  http://www.dynarchlib.com/dl/index.html
  http://www.ymacs.org/

  Fişierul de faţă este o adaptare a fişierului "test.js" din arhiva descărcată de la //ymacs.org,
  în scopul de a putea încărca / salva un buffer Ymacs folosind "yload.php" şi "ysave.php"

  Se presupune creat un VirtualHost conţinând directorul Ymacs (obţinut după dezarhivare).
*/

function saveBuffer() {
    var buf = ymacs.getActiveBuffer();
    var name = prompt("Nume: ", buf.name);
    var rpc = new DlRPC({
        url: "/yphp/ysave.php",
        data: {
            name:  name,
            content: buf.getCode()
        }
    });
    ymacs.renameBuffer(buf, name);
    // alert(rpc.data['name'] + '\n' + rpc.data['content']);
    rpc.call();
};

function showHTML(html, title) {
    var dlg = new DlDialog({
        modal: true,
        resizable: true,
        title: title,
        quitBtn: "destroy"
    });
    var cont = new DlContainer({
        parent: dlg,
        fillParent: true,
        scroll: true
    });
    cont.setContent(html || ymacs.getActiveBuffer().getCode());
    dlg.setSize({ x: 640, y: 480 });
    dlg.show(true);
};

function showHelp() {
    new DlRPC({url:"../../devapp/Help.html"}).call({
        callback: function(data) {
            showHTML(data.text, "Help");
        }
    });
};

DEFINE_SINGLETON("Ymacs_Keymap_ROEmacs", Ymacs_Keymap, function(D, P){

    D.KEYS = {
        // litere mici
        "C-\\ a && C-\\ C-a" : [ "insert", "ă" ],
        "C-\\ q && C-\\ C-q" : [ "insert", "â" ],
        "C-\\ t && C-\\ C-t" : [ "insert", "ţ" ],
        "C-\\ i && C-\\ C-i" : [ "insert", "î" ],
        "C-\\ s && C-\\ C-s" : [ "insert", "ş" ],
        // majuscule
        "C-\\ S-a && C-\\ C-S-a" : [ "insert", "Ă" ],
        "C-\\ S-q && C-\\ C-S-q" : [ "insert", "Â" ],
        "C-\\ S-t && C-\\ C-S-t" : [ "insert", "Ţ" ],
        "C-\\ S-i && C-\\ C-S-i" : [ "insert", "Î" ],
        "C-\\ S-s && C-\\ C-S-s" : [ "insert", "Ş" ],

        "C-x C-s" : saveBuffer,
        "C-c C-c" : showHTML.$C(null, "Preview")
    };

});

Ymacs_Buffer.prototype.makeDefaultKeymap = function() {
    this.pushKeymap(Ymacs_Keymap_ROEmacs());
    return Ymacs_Keymap_Emacs();
};

var desktop = new DlDesktop({});
desktop.fullScreen();

try {
    var dlg = new DlDialog({ title: "Ymacs", resizable: true });

    var layout = new DlLayout({ parent: dlg });

    var ymacs = window.ymacs = new Ymacs({ buffers: [ ], className: "Ymacs-Theme-dark" });

    var menu = new DlHMenu({}); // bară de meniu orizontală
    menu.setStyle({ marginLeft: 0, marginRight: 0 });

    // opţiunea "Load" va lista fişierele dintre care se va selecta unul pentru încărcare în bufferul curent
    var item = new DlMenuItem({ parent: menu, label: "Load ".makeLabel() }); 
        var files = []; // conţinutul subdirectorului Ymacs/devapp

    item.setMenu(function(){
        var menu = new DlVMenu({}); // creează un meniu vertical
        
        // iniţiază un apel RPC "remote procedure call", obţinând de la funcţia "yload.php" şi 
        // înscriind în meniul vertical creat, lista fişierelor din Ymacs/devapp
        
        new DlRPC({ url: "/yphp/yload.php", method: "POST" }).call({ callback: function(data){
            menu.destroyChildWidgets(); //alert("files:"+data.text.split(/\s+/).length);
            data.text.trim().split(/\s+/).sort().foreach(function(file){
                var item = new DlMenuItem({ label: file, parent: menu });
                item.addEventListener("onSelect", function(){
                    // la click pe un item din listă, se va obţine prin DlRPC() conţinutul actual al fişierului
                    var request = new DlRPC({ url: "../../devapp/" + file + "?killCache=" + new Date().getTime() });
                    request.call({
                        callback: function(data){
                            var code = data.text;
                            var buf = ymacs.getBuffer(file) || ymacs.createBuffer({ name: file });
                            buf.setCode(code);
                            // asociază bufferului un "mod de lucru"
                            // pentru PHP trebuie creat unul, dar e bun şi "xml_mode" 
                            if(/(html|css|php)$/.test(file)) buf.cmd("xml_mode", true);
                            else if (/\.(js|c|cpp|cc)$/.test(file)) buf.cmd("javascript_dl_mode", true);
                            
                            ymacs.switchToBuffer(buf);
                        }
                    });
                });
            });
        }});
            return menu;
    });


    var item = new DlMenuItem({ parent: menu, label: "Indentation".makeLabel() });
    item.addEventListener("onSelect", function() {
        var buf = ymacs.getActiveBuffer(), newIndent;
        newIndent = prompt("Indentation level for the current buffer: ", buf.getq("indent_level"));
        if (newIndent != null)
            newIndent = parseInt(newIndent, 10);
        if (newIndent != null && !isNaN(newIndent)) {
            buf.setq("indent_level", newIndent);
            buf.signalInfo("Done setting indentation level to " + newIndent);
        }
    });

    var item = new DlMenuItem({ parent: menu, label: "Save".makeLabel() });
    item.addEventListener("onSelect", saveBuffer);

    var item = new DlMenuItem({ parent: menu, label: "HELP".makeLabel() });
    item.addEventListener("onSelect", showHelp);
    
    menu.addFiller();

    var item = new DlMenuItem({ parent: menu, label: "Toggle line numbers".makeLabel() });
    item.addEventListener("onSelect", function() {
            ymacs.getActiveBuffer().cmd("toggle_line_numbers");
    });

    /* -----[ color theme ]----- */

    var item = new DlMenuItem({ parent: menu, label: "Color theme".makeLabel() });
    var submenu = new DlVMenu({});
    item.setMenu(submenu);

    [
        "dark|Dark background",
        "light|Light background"

    ].foreach(function(theme){
        theme = theme.split(/\|/);
        var item = new DlMenuItem({ parent: submenu, label: theme[1] });
        item.addEventListener("onSelect", ymacs.setColorTheme.$(ymacs, theme[0]));
    });

    /* -----[ font ]----- */

    var item = new DlMenuItem({ parent: menu, label: "Font family".makeLabel() });
    var submenu = new DlVMenu({});
    item.setMenu(submenu);

    item = new DlMenuItem({ parent: submenu, label: "Default from ymacs.css" });
    item.addEventListener("onSelect", function(){
        ymacs.getActiveFrame().setStyle({ fontFamily: "" });
    });

    submenu.addSeparator();

        [
            "Lucida Sans Typewriter",
            "Andale Mono",
            "Courier New",
            "Arial",
            "Verdana",
            "Tahoma",
            "Georgia",
            "Times New Roman"

        ].foreach(function(font){
            item = new DlMenuItem({ parent: submenu, label: "<span style='font-family:" + font + "'>" + font + "</span>" });
            item.addEventListener("onSelect", function(){
                ymacs.getActiveFrame().setStyle({ fontFamily: font });
            });
        });

    /* -----[ font size ]----- */

    var item = new DlMenuItem({ parent: menu, label: "Font size".makeLabel() });
    var submenu = new DlVMenu({});
    item.setMenu(submenu);

    item = new DlMenuItem({ parent: submenu, label: "Default from ymacs.css" });
    item.addEventListener("onSelect", function(){
        ymacs.getActiveFrame().setStyle({ fontSize: "" });
    });

    submenu.addSeparator();

    [
        "11px",
        "12px",
        "14px",
        "16px",
        "18px",
        "20px",
        "22px",
        "24px"

    ].foreach(function(font){
        item = new DlMenuItem({ parent: submenu, label: "<span style='font-size:" + font + "'>" + font + "</span>" });
        item.addEventListener("onSelect", function(){
            ymacs.getActiveFrame().setStyle({ fontSize: font });
        });
    });

    layout.packWidget(menu, { pos: "top" });
    layout.packWidget(ymacs, { pos: "bottom", fill: "*" });

    dlg._focusedWidget = ymacs;
    dlg.setSize({ x: 800, y: 600 });

    // show two frames initially
    // ymacs.getActiveFrame().hsplit();

    dlg.show(true);
    dlg.maximize(true);

} catch(ex) {
    console.log(ex);
}

DynarchDomUtils.trash($("x-loading"));


/*
//
// facem cu un form temporar.

var  form = DynarchDomUtils.createElement("form", {
position : "absolute",
left     : 0,
top      : 0,
zIndex   : -1
    }, {
    id    : "SAVEFORM",
    method: "POST",
    action: "/yphp/ysave.php"
    });
    //     // numele
    DynarchDomUtils.createElement("input", null, {
    type: "hidden",
    name: "name",
    value: prompt("Name: ", ymacs.getActiveBuffer().name) // ymacs.getActiveBuffer().name
    }, form);
    //     // conţinutul
    DynarchDomUtils.createElement("input", null, {
    type: "hidden",
    name: "data",
    value: ymacs.getActiveBuffer().getCode()
    }, form);
    //        }
    //        else {
    //     // refolosim formul dacă era deja făcut
    //            form.elements.namedItem("name").value = ymacs.getActiveBuffer().name;
    //            form.elements.namedItem("data").value = ymacs.getActiveBuffer().getCode();
    //        }

    document.body.appendChild(form);
    if(form.elements.namedItem("name").value) form.submit();
*/
